import argparse
import logging
import queue
import threading


class CommonThread(threading.Thread):

    def __init__(self, *args, **kwargs):
        threading.Thread.__init__(self)
        self.args = args
        self.kwargs = kwargs
        self.inq = queue.Queue()
        self.outq = queue.Queue()
        self.parser = argparse.ArgumentParser()
        self.params = None

    def add_argument(self, *args, **kwargs):
        self.parser.add_argument(*args, **kwargs)

    def parse_args(self):
        str_array = []
        for p in self.args:
            str_array.append(str(p))
        self.params = self.parser.parse_args(str_array)

    def output(self, item, block=True, timeout=None):
        return self.outq.put(item, block, timeout)

    def input(self, block=True, timeout=None):
        return self.outq.get(block, timeout)

    def log_debug(self, msg):
        return logging.debug(msg)

    @classmethod
    def set_basic_logging(cls, level=logging.DEBUG, format='%(threadName)s: %(message)s'):
        logging.basicConfig(level=level, format=format)

    @classmethod
    def some_are_active(cls):
        for thread in threading.enumerate():
            if isinstance(thread, CommonThread):
                return True
        return False

    @classmethod
    def collect_threads_output(cls):
        result = []
        for thread in threading.enumerate():
            if not isinstance(thread, CommonThread):
                continue
            while not thread.outq.empty():
                result.append(thread.outq.get())
        return result

    @classmethod
    def log_threads_output(cls, use_print=False):
        msg_list = CommonThread.collect_threads_output()
        for msg in msg_list:
            if use_print:
                print(msg)
            else:
                logging.debug(msg)


class WorkerThread(CommonThread):

    def __init__(self, worker_function, *args, **kwargs):
        CommonThread.__init__(self, *args, **kwargs)
        self.worker_function = worker_function

    def run(self):
        self.worker_function(self, *self.args, **self.kwargs)
        return None
